/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/* 
 * To see the console logs logged by this file, in the console view, enter the following:
 * gc.console.setCookie("regmap.js", <value>) 
 * Where value: 0 = remove the cookie, 1 = only errors, 2 = errors and warnings, 3 = errors,warnings,info, 4 = errors,warnings,info,logs, 5=debug+all others
 * For more help on how to work with gc.console, please see the help text that is sent to the
 * console when the app starts.
 */
if (!gc.designer) { gc.console.setCookie("regmap.js", 2); }



var saveAsFilePath; // undefined initially

/*
 *  Boilerplate code for creating computed data bindings
 */
document.addEventListener('gc-databind-ready', function() {
    /*
     *   Add custom computed value databindings here, using the following method:
     *
     *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
     *
     *   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
     *   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
     *   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
     */

    // For example, a simple computed values based on simple expression
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");

    // Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
    // gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });

    // Event 1 to n bindings
    /*
    gc.databind.registry.bind('widget.date.value',
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        },
        // getter for date computation
        function(values)
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    );
	*/
});

/*
 *  Boilerplate code for creating custom actions
 */
document.addEventListener('gc-nav-ready', function() {
    /*
     *   Add custom actions for menu items using the following api:
     *
     *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
     *
     *   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
     *   param runable - function that performs the custom action.
     *   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
     *   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
     */

    // For example,
    // gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });

    // Alternatively, to programmatically disable a menu action at any time use:
    // gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction');

/*
    gc.nav.registerAction('show_introduction', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 0;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    
    gc.nav.registerAction('show_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 1;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
    
    
    gc.nav.registerAction('show_motor_control', function() {
        if ((templateObj) && (templateObj.$)) {
            templateObj.$.ti_widget_vtabcontainer.selectedIndex = 2;
            templateObj.$.ti_widget_menuitem.hideFullMenu();
        }
    }, function() {
        return true;
    }, function() {
        return true;
    });
*/

    // regmapTabActive = function() {
    //     let isActive = (document.querySelector("#ti_widget_vtabcontainer").selectedIndex === 4);
    //     console.log("Regmap tab is active?: " + isActive.toString());
    //     return isActive;
    // };
            
    gc.nav.registerAction('load_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            gc.File.browseAndLoad(null, {
                bin: false
            }, function(contents, properties) {
                var regData = null;
                try {
                    regData = JSON.parse(contents);
                    if (regData) {
                        templateObj.setRegisterData(regData);
                        templateObj.$.ti_widget_eventlog_view.log("info", "Load Registers: loaded from :" + properties.localPath);
                    }
                } catch (ex) {
                    templateObj.logMsgAndNotify("error", "Load Registers: Could not parse file:" + ex, false, true);
                }

            });
        }
    }, function() {
        return true;
    }, function() {
        return true; //regmapTabActive(); TODO: Only show menu action when on regmap tab
    });

    // Example of how to save text to file // TODO: Move
    // NOTE: Requires a link to "components/ti-core-assets/ti-core-assets.html" in index.gui
    var saveText = function(fname, text, mime) {
        var blob = new Blob([text], {
            type: mime
        });
        saveAs(blob, fname);
        return false;
    };


    var saveRegsAs = function(calledBy, regDataJson) {
        gc.File.browseAndSave(regDataJson, 'regs.json', '.json', function(result, errorInfo) {
            if (errorInfo) {
                templateObj.logMsgAndNotify("error", calledBy + " " + errorInfo, false, true);
            } else if ((result) && (result.localPath)) {
                saveAsFilePath = result.localPath;
                templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + result.localPath);
            }
        });
    };

    gc.nav.registerAction('save_registers', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                if (!saveAsFilePath) {
                    saveRegsAs("Save Registers", regDataJson);
                } else {
                    gc.File.save(regDataJson, {"localPath": saveAsFilePath}, null, function(errorInfo) {
                        if (errorInfo) {
                            templateObj.logMsgAndNotify("error", "Save Registers As: " + errorInfo, false, true);
                        } else {
                            templateObj.$.ti_widget_eventlog_view.log("info", "Registers saved to " + saveAsFilePath);
                        }
                    });
                }

            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true; //regmapTabActive(); TODO: Only show menu action when on regmap tab
    });

    // TODO: What is the difference between (or need for) SAVE and SAVE AS....
    gc.nav.registerAction('save_registers_as', function() {
        if ((templateObj) && (templateObj.$)) {
            var regData = templateObj.getRegisterData();
            var regDataJson = JSON.stringify(regData);
            if (gc.desktop.isDesktop()) {
                saveRegsAs("Save Registers As", regDataJson);
            } else {
                saveText("regs.json", regDataJson, 'text/plain;charset=utf-8');
                templateObj.$.ti_widget_eventlog_view.log("info", "Save Registers As: regs.json file downloaded");
            }
        }
    }, function() {
        return true;
    }, function() {
        return true; //regmapTabActive(); TODO: Only show menu action when on regmap tab
    });
    
    //gc.nav.disableAction('save_registers_as'); 

});




/*
 *  Boilerplate code for working with components in the application gist
 */
var initComplete = false;
var templateObj;
var registerModel;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    
    // Save reference to template object - TODO: Determine if this is global, and delete duplicate statements
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
    templateObj.addEventListener('dom-change', function() {
        if (initComplete) return;
        this.async(function() {
            initComplete = true;
            gc.console.debug("regmap.js", "Application template has been stamped.");
            
            registerModel = Polymer.dom(document).querySelector('ti-model-register');
            if (registerModel) {
                registerModel.addEventListener("usbHidModelReceivingData", function (event) {
                    templateObj.logMsgAndNotify("debug", "Receiving data from target.", false, false);
                });
            } else {
                gc.console.error("regmap.js", "no ti-model-register component found in this application!");
            }
            
            // Save reference to '#ti_widget_register_page' component
            let regPage = document.querySelector('#ti_widget_register_page'); // Alt: templateObj.$.ti_widget_register_page
            
            if (regPage) {
                
                // get the active device name from the register page
                templateObj.$.ti_widget_menubar.productName = regPage.deviceName;     // TODO: Fix me...The deviceName is not valid yet!
                //templateObj.$.ti_widget_menubar.productName = "ADSxxxx";

                
                // called when the user clicks the Write Register button:
                regPage.addEventListener("write_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));
                
                // called when the user clicks the Write All Registers button:
                regPage.addEventListener("write_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));                
                
                // called when the user clicks the Read Register button:
                regPage.addEventListener("read_reg", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));
                
                // called when the user clicks the Read All Registers button:
                regPage.addEventListener("read_regs", function (event) {
                    var detailsObj = event.detail;
                    templateObj.logMsgAndNotify(detailsObj.statusType, detailsObj.statusMsg, false, true);
                }.bind(this));
                
                // called when the user changes the immediate / deferred mode
                regPage.addEventListener("update_mode_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change update mode to " + detailsObj.mode.label);
                }.bind(this));

                // called when the user changes the auto read refresh rate
                regPage.addEventListener("auto_read_config_changed", function (event) {
                    var detailsObj = event.detail;
                    templateObj.$.ti_widget_eventlog_view.log("info", "request to change automatic read interval to " + detailsObj.autoRead.label);
                }.bind(this));
            }
            
            // TODO: Get from flattened CustomCodec register data?
            templateObj.getRegisterData = function() {
                var regData = {
                    signature: "oneui-register-data",
                    data: []
                };
                var regBlocks = regPage.regBlocks;
                for (var i = 0; i < regBlocks.length; i++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[i].registers.length; r++) {
                        var reg = regBlocks[i].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        var item = {
                            idx: r,
                            id: regName,
                            value: reg.value
                        };
                        registersData.push(item);
                    }
                    regData.data.push(registersData);
                }
                var strJson = JSON.stringify(regData);
                return strJson;
            };
            
            templateObj.setRegisterData = function(strJson) {
                var regData = JSON.parse(strJson);
                var ok = true;
                var regBlocks = regPage._registerView.regBlocks;
                for (var b = 0; b < regBlocks.length; b++) {
                    var registersData = [];
                    for (var r = 0; r < regBlocks[b].registers.length; r++) {
                        var reg = regBlocks[b].registers[r];
                        var regName = reg.name.toLowerCase();
                        regName = regName.replace(/\&/g, "_");
                        item = regData.data[b][r];
                        if (item.id === regName) {
                            reg.value = item.value;
                            if (!regPage.isDeferredWrite) {
                                reg.valueLastWritten = reg.value;
                                var intValue = gc.utils.getValueFromHexString("" + reg.value);
                                regPage.writeValueToTargetRegister(intValue, null, b, r);
                            } else {
                                regPage._registerView.refreshRegValue(b,r);
                            }
                        } else {
                            var msg = "load registers failed: register name mismatch (" + item.id + " does not match " + regName + ")";
                            templateObj.logMsgAndNotify("warning", msg, false, true);
                        }
                    }
                }
                if (regPage.isDeferredWrite){
                    regPage._registerView._deferredAccessCtr++; // update the page
                }
            };
        }, 1);
    });
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded', init);
}